//
//  KMViewController.m
//  KyotoMap
//
//  Created by kunii on 2013/01/17.
//  Copyright (c) 2013年 國居貴浩. All rights reserved.
//

#import <MapKit/MapKit.h>
#import "KMViewController.h"

@interface KMAreaAnnotation : MKPointAnnotation
@end
@implementation KMAreaAnnotation
@end

@interface KMViewController ()<MKMapViewDelegate> {
    MKMapView* _mapView;
    NSArray*    _annotations;
    NSArray*    _areaannotations;
}
@end

@implementation KMViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    _mapView = [[MKMapView alloc] initWithFrame:self.view.frame];
    _mapView.delegate = self;
    [self.view addSubview:_mapView];
    //  京都　latitude：35.0212466 longitude：135.7555968
    CLLocationCoordinate2D center = CLLocationCoordinate2DMake(35.0212466, 135.7555968);
    MKCoordinateRegion kyotoregion = MKCoordinateRegionMakeWithDistance(center,
                                                                        10000.0,  //  10km
                                                                        10000.0);
    _mapView.region = kyotoregion;  //  アニメーション抜き

    //  2秒遅らせる
    int64_t delayInSeconds = 2.0;
    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, delayInSeconds * NSEC_PER_SEC);
    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
        //  1000個のピンを落とす
        NSMutableArray* array = [NSMutableArray arrayWithCapacity:1000];
        for (int i = 0; i < 1000; i++) {
            MKPointAnnotation* pin = [[MKPointAnnotation alloc] init];
            pin.coordinate = CLLocationCoordinate2DMake(
                                                        35.0212466 + (float)((rand() % 100) - 50) / 100.0 * 0.1,
                                                        135.7555968 + (float)((rand() % 100) - 50) / 100.0 * 0.1);
//            [_mapView addAnnotation:pin];
            [array addObject:pin];
        }
        _annotations = array;   //  1000個のMKPointAnnotationを持ったNSArray
        //  特定間隔で、間引き
        NSMutableDictionary* dic = [NSMutableDictionary dictionaryWithCapacity:[_annotations count]];
        for (MKPointAnnotation* pin in _annotations) {
            NSString* key = [NSString stringWithFormat:@"%.2fx%.2f", pin.coordinate.latitude / 2, pin.coordinate.longitude / 2];
            KMAreaAnnotation* area = [dic valueForKey:key];
            if (area == nil) {
                area = [[KMAreaAnnotation alloc] init];
                area.coordinate = pin.coordinate;
                [dic setObject:area forKey:key];
            }
        }
        _areaannotations = [dic allValues];  //  間引かれたKMAreaAnnotationを持ったNSArray
        [self mapView:_mapView regionDidChangeAnimated:NO]; //  ピンを画面に表示
    });

    //  長押し見張り
    UILongPressGestureRecognizer* tapGestureRecognizer = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(tap:)];
    [_mapView addGestureRecognizer:tapGestureRecognizer];
}

//  地図の表示エリアが変わった
- (void)mapView:(MKMapView *)mapView
regionDidChangeAnimated:(BOOL)animated
{
    NSMutableSet* addset = nil;
    if (mapView.region.span.latitudeDelta > 0.02) {
        addset = [NSMutableSet setWithArray:_areaannotations];
    } else {
        addset = [NSMutableSet setWithArray:_annotations];
    }
    NSMutableSet* removeset = [NSMutableSet setWithArray:mapView.annotations];
    NSSet* alreadyexistset = [removeset copy];
    [removeset minusSet:addset];                        //  追加対象側に存在するものは、削除対象からはぶく
    [addset minusSet:alreadyexistset];                  //  すでにmapView.annotations側に存在するものは追加対象からはぶく
    [mapView removeAnnotations:removeset.allObjects];   //  削除
    [mapView addAnnotations:addset.allObjects];         //  追加
}

//  地図タップ対応
- (void)tap:(UILongPressGestureRecognizer*)gestureRecognizer
{
    if (gestureRecognizer.state != UIGestureRecognizerStateBegan)
        return;
    CGPoint pt = [gestureRecognizer locationInView:gestureRecognizer.view];
    CLLocationCoordinate2D coordinate = [_mapView convertPoint:pt toCoordinateFromView:gestureRecognizer.view];    
    MKPointAnnotation* pin = [[MKPointAnnotation alloc] init];
    pin.title = @"京都";
    pin.coordinate = coordinate;
    [_mapView addAnnotation:pin];
}

- (MKAnnotationView *)mapView:(MKMapView *)mapView viewForAnnotation:(id <MKAnnotation>)annotation
{
    if ([annotation isKindOfClass:[KMAreaAnnotation class]]) {
        static NSString* Identifier = @"AreaAnnotationIdentifier";
        MKAnnotationView* aView = (MKAnnotationView *)[mapView dequeueReusableAnnotationViewWithIdentifier:Identifier];
        if (aView == nil) {
            aView = [[MKAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:Identifier];
            aView.bounds = CGRectMake(0,0,16, 16);
            aView.backgroundColor = [UIColor orangeColor];
            aView.canShowCallout = NO;
            aView.draggable = NO;
            return aView;
        }
        aView.annotation = annotation;
        return aView;
    }
    static NSString* Identifier = @"PinAnnotationIdentifier";
    MKPinAnnotationView* pinView = (MKPinAnnotationView *)[mapView dequeueReusableAnnotationViewWithIdentifier:Identifier];
    if (pinView == nil) {
        pinView = [[MKPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:Identifier];
        pinView.animatesDrop = YES;
        pinView.canShowCallout = YES;
        pinView.draggable = YES;
        //  デスクロージャ追加
        UIButton* rightButton = [UIButton buttonWithType:UIButtonTypeDetailDisclosure];
        pinView.rightCalloutAccessoryView = rightButton;
        return pinView;
    }
    pinView.annotation = annotation;
    return pinView;
}

//  デスクロージャタップ対応
- (void)mapView:(MKMapView *)mapView annotationView:(MKAnnotationView *)view calloutAccessoryControlTapped:(UIControl *)control
{
    printf("tap DetailDisclosure\n");
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

@end
